// Package nl80211 is an auto-generated package which contains constants and
// types used to access nl80211 information using generic netlink.
//
// Special thanks to Maxim Kupriianov for the cgogen tool, which makes the
// automatic generation of Go constants possible.
//
//  https://github.com/xlab/cgogen
//
// Maxim was also kind enough to provide the cgogen example configuration which
// was used to create this package.
//
//  https://github.com/xlab/nl80211
package nl80211

//go:generate cgogen -out ../ -nocgo nl80211.yml
